{% comment ******************* %}
{% comment document header     %}
{% comment ******************* %}
<header>
    <h2>{{ doc.ext.full_display_name }}
    </h2>
    {% block doc-nav %}{% end %}
</header>

{% comment ***************** %}
{% comment document content  %}
{% comment ***************** %}
{% block doc-content %}{% end %}

{% comment ******************* %}
{% comment document footer     %}
{% comment ******************* %}
<footer>
    <h4>
		<span>
	        ES-DOC Documentation Viewer (v{{ ".".join(pyesdoc.__version__.split(".")[0:-1]) }}) &copy; ES-DOC 2016
		</span>
    </h4>
</footer>
