{% if requirements %}
<section>
	<header>
		<h{{ header }}>{{ description }}</h{{ header }}>
	</header>
	{% set processed_requirements = [] %}
	{% for requirement in requirements %}
	{% if requirement.long_name not in processed_requirements %}
	{% set ctx = TemplateInfo(requirement, fieldset=str(type(requirement))) %}
	<section>
		<header>
			<h{{ header + 1 }}>{{ requirement.long_name }}</h{{ header + 1 }}>
		</header>
		<table>
			<tbody>
			{% include ../core/fieldset.html %}
			</tbody>
		</table>
	</section>
	{% set processed_requirements += [requirement.long_name] %}
	{% end %}
	{% end %}
</section>
{% end %}
