
# Specialization type keys.
TYPE_KEY_ENUM = 'enum'
TYPE_KEY_ENUM_CHOICE = 'enum-choice'
TYPE_KEY_GRID = "grid"
TYPE_KEY_KEYPROPS = "keyprops"
TYPE_KEY_MODEL = "model"
TYPE_KEY_PROCESS = "process"
TYPE_KEY_PROPERTY = "property"
TYPE_KEY_PROPERTY_SET = "property-set"
TYPE_KEY_REALM = "realm"
TYPE_KEY_SUBPROCESS = "subprocess"
