# -*- coding: utf-8 -*-
"""
.. module:: cim.__init__.py

   :license: GPL / CeCILL
   :platform: Unix, Windows
   :synopsis: The cim package initializer.

.. moduleauthor:: Earth System Documentation (ES-DOC) <dev@es-doc.org>
.. note:: Code generated using pyesdoc @ 2013-04-17 13:14:13.903808.

"""
import v1
import v2
import constants


# Supported version of this ontology.
VERSIONS = (v1, v2)
