<section>
	<header>
		<h{{ header }}>Citations</h{{ header }}>
	</header>
	{% for citation in citations %}
	{% set ctx = TemplateInfo(citation, fieldset='cim.2.shared.citation') %}
	<section>
		<header>
			<h{{ header + 1 }}>{{ citation.title }}</h{{ header + 1}}>
		</header>
		<table>
			<tbody>
			{% include ../core/fieldset.html %}
			</tbody>
		</table>
	</section>
	{% end %}
</section>