<section>
	<header>
		<h5>{{ member.ensemble_ids[0].value }}</h5>
	</header>
	<table>
		<tbody>
		{% set ctx = TemplateInfo(member, fieldset=("cim.1.activity.ensemble-member", member)) %}
		{% include ../core/fieldset.html %}
		</tbody>
	</table>
</section>

