{% set mods = [c for c in doc.model.changes if len(c.details)] %}
{% if mods %}
<section id="{{ doc.ext.full_id }}-model-mods">
	<header>
		<h3>Model Modifications</h3>
	</header>
	<table>
		<tbody>
		{% set ctx = TemplateInfo(doc, fieldset_type='property', fieldset=("cim.1.activity.simulationrun-model-mods", mods)) %}
		{% include ../core/fieldset.html %}
		</tbody>
	</table>
</section>
{% end %}
