<table>
	{% comment ***************** %}
	{% comment Emit overview     %}
	{% comment ***************** %}
	<tbody>
		<tr>
			<td class="esdoc-field-name"><b>{{requirement.name}}</b></td>
			<td class="esdoc-field-subname">{{requirement.id if requirement.id else ""}}</td>
			<td>{{requirement.description}}</td>
		</tr>

	{% comment ***************** %}
	{% comment Emit options      %}
	{% comment ***************** %}
	{% set options = [o for o in requirement.options if o] %}
	{% if options %}
	{% set options = sorted(options, key=lambda o: o.name.lower() if o.name else str()) %}
	{% for option in options %}
		<tr>
			<td class="esdoc-field-name"><b>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;{{option.name}}</b></td>
			<td class="esdoc-field-subname">{{option.id}}</td>
			<td>{{option.description}}</td>
		</tr>
	{% end %}
	{% end %}
	</tbody>
</table>